@@ -13,12 +13,12 @@
 #ifndef XBUILDER_HPP
 #define XBUILDER_HPP
 
-#include <utility>
-#include <functional>
 #include <cmath>
+#include <functional>
+#include <utility>
 
-#include "xfunction.hpp"
 #include "xbroadcast.hpp"
+#include "xfunction.hpp"
 #include "xgenerator.hpp"
 
 #ifdef X_OLD_CLANG
@@ -53,7 +53,7 @@ namespace xt
     }
 #else
     template <class T, class I, std::size_t L>
-    inline auto ones(const I(&shape)[L]) noexcept
+    inline auto ones(const I (&shape)[L]) noexcept
     {
         return broadcast(T(1), shape);
     }
@@ -72,7 +72,7 @@ namespace xt
     {
         return broadcast(T(0), std::forward<S>(shape));
     }
-    
+
 #ifdef X_OLD_CLANG
     template <class T, class I>
     inline auto zeros(std::initializer_list<I> shape) noexcept
@@ -81,7 +81,7 @@ namespace xt
     }
 #else
     template <class T, class I, std::size_t L>
-    inline auto zeros(const I(&shape)[L]) noexcept
+    inline auto zeros(const I (&shape)[L]) noexcept
     {
         return broadcast(T(0), shape);
     }
@@ -90,8 +90,10 @@ namespace xt
     namespace detail
     {
         template <class T>
-        struct arange_impl
+        class arange_impl
         {
+        public:
+
             using value_type = T;
 
             arange_impl(T start, T stop, T step)
@@ -112,6 +114,7 @@ namespace xt
             }
 
         private:
+
             value_type m_start;
             value_type m_stop;
             value_type m_step;
@@ -129,19 +132,22 @@ namespace xt
         };
 
         template <class F>
-        struct fn_impl
+        class fn_impl
         {
+        public:
+
             using value_type = typename F::value_type;
             using size_type = std::size_t;
 
-            fn_impl(F&& f) : m_ft(f)
+            fn_impl(F&& f)
+                : m_ft(f)
             {
             }
 
             template <class... Args>
             inline value_type operator()(Args... args) const
             {
-                size_type idx [sizeof...(Args)] = {static_cast<size_type>(args)...};
+                size_type idx[sizeof...(Args)] = {static_cast<size_type>(args)...};
                 return access_impl(std::begin(idx), std::end(idx));
             }
 
@@ -161,11 +167,14 @@ namespace xt
         };
 
         template <class T>
-        struct eye_fn
+        class eye_fn
         {
+        public:
+
             using value_type = T;
 
-            eye_fn(int k) : m_k(k)
+            eye_fn(int k)
+                : m_k(k)
             {
             }
 
@@ -273,8 +282,10 @@ namespace xt
     namespace detail
     {
         template <class... CT>
-        struct concatenate_impl
+        class concatenate_impl
         {
+        public:
+
             using size_type = std::size_t;
             using value_type = std::common_type_t<typename std::decay_t<CT>::value_type...>;
 
@@ -319,7 +330,7 @@ namespace xt
                 std::size_t i = 0;
                 for (; i < sizeof...(CT); ++i)
                 {
-                    if (apply<bool>(i, match, m_t)) 
+                    if (apply<bool>(i, match, m_t))
                     {
                         break;
                     }
@@ -332,8 +343,10 @@ namespace xt
         };
 
         template <class... CT>
-        struct stack_impl
+        class stack_impl
         {
+        public:
+
             using size_type = std::size_t;
             using value_type = std::common_type_t<typename std::decay_t<CT>::value_type...>;
 
@@ -374,14 +387,17 @@ namespace xt
         };
 
         template <class CT>
-        struct repeat_impl
+        class repeat_impl
         {
+        public:
+
             using xexpression_type = std::decay_t<CT>;
             using size_type = typename xexpression_type::size_type;
             using value_type = typename xexpression_type::value_type;
 
-            repeat_impl(CT source, size_type axis) :
-                m_source(source), m_axis(axis)
+            template <class CTA>
+            repeat_impl(CTA&& source, size_type axis)
+                : m_source(std::forward<CTA>(source)), m_axis(axis)
             {
             }
 
@@ -531,13 +547,16 @@ namespace xt
     namespace detail
     {
         template <class CT>
-        struct diagonal_fn
+        class diagonal_fn
         {
+        public:
+
             using xexpression_type = std::decay_t<CT>;
             using value_type = typename xexpression_type::value_type;
 
-            diagonal_fn(CT source, int offset, std::size_t axis_1, std::size_t axis_2)
-                : m_source(source), m_offset(offset), m_axis_1(axis_1), m_axis_2(axis_2)
+            template <class CTA>
+            diagonal_fn(CTA&& source, int offset, std::size_t axis_1, std::size_t axis_2)
+                : m_source(std::forward<CTA>(source)), m_offset(offset), m_axis_1(axis_1), m_axis_2(axis_2)
             {
             }
 
@@ -567,6 +586,7 @@ namespace xt
             }
 
         private:
+
             CT m_source;
             const int m_offset;
             const std::size_t m_axis_1;
@@ -574,12 +594,16 @@ namespace xt
         };
 
         template <class CT>
-        struct diag_fn
+        class diag_fn
         {
+        public:
+
             using xexpression_type = std::decay_t<CT>;
             using value_type = typename xexpression_type::value_type;
 
-            diag_fn(CT source, int k) : m_source(source), m_k(k)
+            template <class CTA>
+            diag_fn(CTA&& source, int k)
+                : m_source(std::forward<CTA>(source)), m_k(k)
             {
             }
 
@@ -597,19 +621,23 @@ namespace xt
             }
 
         private:
+
             CT m_source;
             const int m_k;
         };
 
         template <class CT>
-        struct flip_impl
+        class flip_impl
         {
+        public:
+
             using xexpression_type = std::decay_t<CT>;
             using value_type = typename xexpression_type::value_type;
             using size_type = typename xexpression_type::size_type;
 
-            flip_impl(CT source, std::size_t axis)
-                : m_source(source), m_axis(axis), m_shape_at_axis(m_source.shape()[m_axis] - 1)
+            template <class CTA>
+            flip_impl(CTA&& source, std::size_t axis)
+                : m_source(std::forward<CTA>(source)), m_axis(axis), m_shape_at_axis(m_source.shape()[m_axis] - 1)
             {
             }
 
@@ -629,6 +657,7 @@ namespace xt
             }
 
         private:
+
             template <class It>
             inline value_type access_impl(It begin, It end) const
             {
@@ -643,14 +672,17 @@ namespace xt
         };
 
         template <class CT, class Comp>
-        struct trilu_fn
+        class trilu_fn
         {
+        public:
+
             using xexpression_type = std::decay_t<CT>;
             using value_type = typename xexpression_type::value_type;
             using signed_idx_type = long int;
 
-            trilu_fn(CT source, int k, Comp comp)
-                : m_source(source), m_k(k), m_comp(comp)
+            template <class CTA>
+            trilu_fn(CTA&& source, int k, Comp comp)
+                : m_source(std::forward<CTA>(source)), m_k(k), m_comp(comp)
             {
             }
 
@@ -662,6 +694,7 @@ namespace xt
             }
 
         private:
+
             CT m_source;
             const signed_idx_type m_k;
             const Comp m_comp;
@@ -736,6 +769,7 @@ namespace xt
      * @brief xexpression with values of arr on the diagonal, zeroes otherwise
      *
      * @param arr the 1D input array of length n
+     * @param k the offset of the considered diagonal
      * @returns xexpression function with shape n x n and arr on the diagonal
      *
      * \code{.cpp}
@@ -751,7 +785,7 @@ namespace xt
         using CT = xclosure_t<E>;
         std::size_t s = arr.shape()[0] + std::abs(k);
         return detail::make_xgenerator(detail::fn_impl<detail::diag_fn<CT>>(detail::diag_fn<CT>(std::forward<E>(arr), k)),
-                                       { s, s });
+                                       {s, s});
     }
 
     /**
@@ -788,7 +822,7 @@ namespace xt
         using CT = xclosure_t<E>;
         auto shape = arr.shape();
         return detail::make_xgenerator(detail::fn_impl<detail::trilu_fn<CT, std::greater_equal<long int>>>(
-                                       detail::trilu_fn<CT, std::greater_equal<long int>>(std::forward<E>(arr), k, std::greater_equal<long int>())),
+                                           detail::trilu_fn<CT, std::greater_equal<long int>>(std::forward<E>(arr), k, std::greater_equal<long int>())),
                                        shape);
     }
 
@@ -807,8 +841,8 @@ namespace xt
         using CT = xclosure_t<E>;
         auto shape = arr.shape();
         return detail::make_xgenerator(detail::fn_impl<detail::trilu_fn<CT, std::less_equal<long int>>>(
-                                       detail::trilu_fn<CT, std::less_equal<long int>>(std::forward<E>(arr), k, std::less_equal<long int>())),
+                                           detail::trilu_fn<CT, std::less_equal<long int>>(std::forward<E>(arr), k, std::less_equal<long int>())),
                                        shape);
     }
 }
-#endif
\ No newline at end of file
+#endif
