@@ -9,14 +9,14 @@
 #ifndef XCONTAINER_HPP
 #define XCONTAINER_HPP
 
-#include <numeric>
 #include <functional>
+#include <numeric>
 
-#include "xstrides.hpp"
 #include "xiterable.hpp"
 #include "xiterator.hpp"
-#include "xoperation.hpp"
 #include "xmath.hpp"
+#include "xoperation.hpp"
+#include "xstrides.hpp"
 #include "xtensor_forward.hpp"
 
 namespace xt
@@ -24,8 +24,12 @@ namespace xt
 
     namespace check_policy
     {
-        struct none {};
-        struct full {};
+        struct none
+        {
+        };
+        struct full
+        {
+        };
     }
 
     template <class D>
@@ -107,7 +111,7 @@ namespace xt
         void transpose(std::initializer_list<I> permutation, Tag check_policy = Tag());
 #else
         template <class I, std::size_t N, class Tag = check_policy::none>
-        void transpose(const I(&permutation)[N], Tag check_policy = Tag());
+        void transpose(const I (&permutation)[N], Tag check_policy = Tag());
 #endif
 
         template <class... Args>
@@ -129,6 +133,10 @@ namespace xt
         container_type& data() noexcept;
         const container_type& data() const noexcept;
 
+        value_type* raw_data() noexcept;
+        const value_type* raw_data() const noexcept;
+        const size_type raw_data_offset() const noexcept;
+
         template <class S>
         bool broadcast_shape(S& shape) const;
 
@@ -362,7 +370,7 @@ namespace xt
 #else
     template <class D>
     template <class I, std::size_t N, class Tag>
-    inline void xcontainer<D>::transpose(const I(&permutation)[N], Tag check_policy)
+    inline void xcontainer<D>::transpose(const I (&permutation)[N], Tag check_policy)
     {
         transpose_impl(permutation, check_policy);
     }
@@ -436,6 +444,7 @@ namespace xt
     template <class... Args>
     inline auto xcontainer<D>::operator()(Args... args) -> reference
     {
+        XTENSOR_ASSERT(check_index(shape(), args...));
         size_type index = data_offset<size_type>(strides(), static_cast<size_type>(args)...);
         return data()[index];
     }
@@ -450,6 +459,7 @@ namespace xt
     template <class... Args>
     inline auto xcontainer<D>::operator()(Args... args) const -> const_reference
     {
+        XTENSOR_ASSERT(check_index(shape(), args...));
         size_type index = data_offset<size_type>(strides(), static_cast<size_type>(args)...);
         return data()[index];
     }
@@ -501,6 +511,7 @@ namespace xt
     template <class It>
     inline auto xcontainer<D>::element(It first, It last) -> reference
     {
+        XTENSOR_ASSERT(check_element_index(shape(), first, last));
         return data()[element_offset<size_type>(strides(), first, last)];
     }
 
@@ -515,6 +526,7 @@ namespace xt
     template <class It>
     inline auto xcontainer<D>::element(It first, It last) const -> const_reference
     {
+        XTENSOR_ASSERT(check_element_index(shape(), first, last));
         return data()[element_offset<size_type>(strides(), first, last)];
     }
 
@@ -536,6 +548,30 @@ namespace xt
     {
         return derived_cast().data_impl();
     }
+
+    /**
+     * Returns the offset to the first element in the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::raw_data() noexcept -> value_type*
+    {
+        return data().data();
+    }
+
+    template <class D>
+    inline auto xcontainer<D>::raw_data() const noexcept -> const value_type*
+    {
+        return data().data();
+    }
+
+    /**
+     * Returns the offset to the first element in the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::raw_data_offset() const noexcept -> const size_type
+    {
+        return size_type(0);
+    }
     //@}
 
     /**
@@ -758,4 +794,3 @@ namespace xt
 }
 
 #endif
-
