@@ -9,16 +9,16 @@
 #ifndef XGENERATOR_HPP
 #define XGENERATOR_HPP
 
+#include <algorithm>
 #include <cstddef>
+#include <numeric>
+#include <tuple>
 #include <type_traits>
 #include <utility>
-#include <tuple>
-#include <algorithm>
-#include <numeric>
 
 #include "xexpression.hpp"
-#include "xstrides.hpp"
 #include "xiterable.hpp"
+#include "xstrides.hpp"
 #include "xutils.hpp"
 
 namespace xt
@@ -185,13 +185,14 @@ namespace xt
     template <class... Args>
     inline auto xgenerator<F, R, S>::operator()(Args... args) const -> const_reference
     {
+        XTENSOR_ASSERT(check_index(shape(), args...));
         return m_f(args...);
     }
 
     template <class F, class R, class S>
     inline auto xgenerator<F, R, S>::operator[](const xindex& index) const -> const_reference
     {
-        return m_f.element(index.begin(), index.end());
+        return element(index.begin(), index.end());
     }
 
     template <class F, class R, class S>
@@ -211,10 +212,11 @@ namespace xt
     template <class It>
     inline auto xgenerator<F, R, S>::element(It first, It last) const -> const_reference
     {
+        XTENSOR_ASSERT(check_element_index(shape(), first, last));
         return m_f.element(first, last);
     }
     //@}
-    
+
     /**
      * @name Broadcasting
      */
@@ -260,7 +262,7 @@ namespace xt
         return const_stepper(this, offset, true);
     }
 
-     namespace detail
+    namespace detail
     {
 #ifdef X_OLD_CLANG
         template <class Functor, class I>
@@ -272,14 +274,14 @@ namespace xt
         }
 #else
         template <class Functor, class I, std::size_t L>
-        inline auto make_xgenerator(Functor&& f, const I(&shape)[L]) noexcept
+        inline auto make_xgenerator(Functor&& f, const I (&shape)[L]) noexcept
         {
             using shape_type = std::array<std::size_t, L>;
             using type = xgenerator<Functor, typename Functor::value_type, shape_type>;
             return type(std::forward<Functor>(f), forward_sequence<shape_type>(shape));
         }
 #endif
-        
+
         template <class Functor, class S>
         inline auto make_xgenerator(Functor&& f, S&& shape) noexcept
         {
