@@ -10,8 +10,8 @@
 #define XTENSOR_CONFIG_HPP
 
 #define XTENSOR_VERSION_MAJOR 0
-#define XTENSOR_VERSION_MINOR 7
-#define XTENSOR_VERSION_PATCH 3
+#define XTENSOR_VERSION_MINOR 8
+#define XTENSOR_VERSION_PATCH 0
 
 // DETECT 3.6 <= clang < 3.8 for compiler bug workaround.
 #ifdef __clang__
@@ -23,7 +23,7 @@
 #endif
 
 #ifndef DEFAULT_DATA_CONTAINER
-#define DEFAULT_DATA_CONTAINER(T, A) std::vector<T, A>
+#define DEFAULT_DATA_CONTAINER(T, A) uvector<T, A>
 #endif
 
 #ifndef DEFAULT_SHAPE_CONTAINER
